////////////////////////////////////////////////////////////////////////////////
// Filename: Automaton.hh                                                     //
// Purpose : header file for call 'Automaton'                                 //
// provides several functions to build a NFA                                  //
////////////////////////////////////////////////////////////////////////////////
// Author: Markus Keidl                                                       //
// Theme.: 2                                                                  //
// Group.: 5 (Gerald Anleitner, Stefan Seltzsam, Markus Keidl)                //
//                                                                            //
// Date of creation: 28.05.1996        Date of last change: 16.07.1996        //
////////////////////////////////////////////////////////////////////////////////

#ifndef _AUTOMATON_HH
#define _AUTOMATON_HH

#include <DLList.h>
#include <iostream.h>
#include <String.h>
#include <fstream.h>
#include "transition.hh"
#include "labelled_transition.hh"
#include "state.hh"
#include "unlabelled_state.hh"
#include "unlabelled_end_state.hh"
#include "Protocol.hh"

class Automaton
{
public:
  Automaton();                       // default constructor
  ~Automaton();			     // destructor

  void add_state(State* new_state);  // add a new State to the Automaton
  void set_start_state(State* new_st_state);
				     // set the start-State of the A.
  void add_transition(State* source_state, Transition* new_trans);
				     // add a Transition to the State source
  void match(Input_Stream& text_to_scan, Protocol& hits);
				     // checks the beginning of text_to_scan to
				     // be a part of the language of the A.
  int create_from_regexp(const String& path);
				     // creates the automaton that spots the
				     // specified language
  void optimize_automaton ();
  bool sgraph(const String& path);    // Xsaves the nfa in the Sgraph-format 

private:
  bool is_valid(String& expression);	     // checks, if regexp is an correct regular expression
  int interpret_regexp(State* start, State* ende, String& regexp);
  String get_next_op(String& regexp);
  int expand_regexp (String& regexp);
  int number_of_state(State* which_state, State** array, int size);

  // functions used by expand_regexp
  int is_valid_quoted(String quote, int in_out);
  int check_regexp(String& regexp);
  int find_block_end (String& regexp, int start);
  String quote (char quote) ;
  int expand_block (String& block);
  int expand_checked_regexp (String& regexp);

  // datamembers
  DLList<State*> state_list;
  State* start_state;
};

#endif






