////////////////////////////////////////////////////////////////////////////////
// Filename: Input_Stream.cc                                                  //
// Purpose.: inplements the class 'Input_Stream', a list of Input_Atom's      //
////////////////////////////////////////////////////////////////////////////////
// Author: Markus Keidl                                                       //
// Theme.: 2                                                                  //
// Group.: 5 (Gerald Anleitner, Stefan Seltzsam, Markus Keidl)                //
//                                                                            //
// Date of creation: 20.05.1996        Date of last change: 21.05.1996        //
////////////////////////////////////////////////////////////////////////////////

#include <DLList.h>
#include <String.h>
#include <fstream.h>

#include "Input_Atom.hh"
#include "Input_Stream.hh"

// default constructor
Input_Stream::Input_Stream() : itr(0) {}

// copy constructor
Input_Stream::Input_Stream(const Input_Stream& rhs) : input_stream(rhs.input_stream), itr(rhs.itr) {}

// destructor
Input_Stream::~Input_Stream() 
{
  input_stream.clear();
}

// adds an Input_Atom to the list
void Input_Stream::add_Input_Atom(const Input_Atom& atom)
{
  input_stream.append(atom);
}

// reads an Inut_Stream from a file, specified by the full file path
int Input_Stream::read_from_file(const String& path)
{
  // every letter from the source file is transferred to the Input_Stream
  ifstream read_from(path);	// open file
  if (read_from == 0)		// maybe file is not existing
    {
      cerr << path << ": no such file found" << endl;
      return false;
    }
  char ch;
  while (read_from.get(ch))
    {
      Input_Atom to_add(ch);	// cast char 'ch' to String
      add_Input_Atom(to_add);	// add Input_Atom to Input_Stream (this)
    }
  if (!read_from.eof())          // I/O error
    {
      cerr << path << ": error occured while reading file" << endl;
      return false;
    }
  return true;
}

// returns if the stream is empty
bool Input_Stream::is_empty()
{
  return input_stream.empty();
}

// cuts off the first anz Input_Atom's
void Input_Stream::cut_off(int anz)
{
  for(int i = 1 ; i <= anz ; i++)
    input_stream.del_front();
}

// Iteratorimplementation

// set iterator to first item
void Input_Stream::first()
{
  itr = input_stream.first();
}

// further Input_Atom's available?
int Input_Stream::operator!() const
{
  return (itr != 0);
}

// get current item
Input_Atom& Input_Stream::operator()()
{
  return input_stream(itr);
}

// set iterator to next item
void Input_Stream::next()
{
  input_stream.next(itr);
}

// get current position in list
Pix Input_Stream::get_itr()
{
  return itr;
}

// set new position in list
void Input_Stream::set_itr(Pix i)
{
  itr = i;
}

ostream& operator<<(ostream& s, Input_Stream& str)
{
  for (Pix i = str.input_stream.first(); i != 0 ; str.input_stream.next(i))
    s << str.input_stream(i);
  return s;
}

