////////////////////////////////////////////////////////////////////////////////
// Filename: Protocol.cc                                                      //
// Purpose: implements the class 'Protocol'                                  //
////////////////////////////////////////////////////////////////////////////////
// Author: Markus Keidl                                                       //
// Theme.: 2                                                                  //
// Group.: 5 (Gerald Anleitner, Stefan Seltzsam, Markus Keidl)                //
//                                                                            //
// Date of Creation: 21.05.1996        Date of last change: 13.07.1996        //
////////////////////////////////////////////////////////////////////////////////

#include "Protocol.hh"

// copy constructor
Protocol::Protocol(const Protocol& rhs) : token(rhs.token), length(rhs.length),
  position(rhs.position), input_stream(rhs.input_stream) {}

// init constructor
Protocol::Protocol(Input_Stream& s) : token(""), length(0), position(0), input_stream(s) {}

// destructor
Protocol::~Protocol()
{}

// get the scanned token
String& Protocol::get_token()
{
  return token;
}

// get the length of the scanned stream
int Protocol::get_length()
{
  return length;
}		

// get Input_Stream
Input_Stream& Protocol::get_Input_Stream() const
{
  return input_stream;
}

// erase all values in Protocol except input_stream
void Protocol::clear()
{
  token = "";
  length = 0;
  position = NULL;
}

// get the scanned Input_Stream
Input_Stream Protocol::get_scanned_stream() const
{
  Input_Stream new_stream;
  Pix i = input_stream.get_itr();
  for (input_stream.first() ; input_stream.get_itr() != position && !input_stream ; input_stream.next())
    new_stream.add_Input_Atom(input_stream());
  input_stream.set_itr(i);
  return new_stream;
}

// get the rest of the Input_Stream
Input_Stream Protocol::get_rest_stream() const
{
  Input_Stream new_stream;
  Pix i = input_stream.get_itr();
  for (input_stream.first() ; input_stream.get_itr() != position && !input_stream ; input_stream.next());
  for ( ; !input_stream ; input_stream.next())
    new_stream.add_Input_Atom(input_stream());
  input_stream.set_itr(i);
  return new_stream;
}

// add hit with token
void Protocol::add_hit(const String& rec_token)
{
  Pix itr_bak = input_stream.get_itr();	// backup current iterator

  // compute the length of the new hit
  int new_len = 0;
  for (input_stream.first() ; input_stream.get_itr() != itr_bak ; input_stream.next())
    new_len++;

  if (new_len > length)
    {
      token = rec_token;
      position = itr_bak;
      length = new_len;
    }
  
}

ostream& operator<<(ostream& s, Protocol prot)
{
  s << "(" << prot.get_token() << ",\"";
  Input_Stream& str = prot.get_Input_Stream();
  Pix i = str.get_itr();
  for (str.first() ; str.get_itr() != prot.position ; str.next())
    {
      if (str().get_Input_Atom() == '\\' || str().get_Input_Atom() == '"')
	s << '\\';
      s << str();
    }
  s << "\")";
  str.set_itr(i);
  return s;
}



